'PSB Paint Demo (C) Copyright 2009 Rayslogic.com, LLC
' A simple paint app demo for the PSB showing graphics and touchscreen interface

CON

  _clkmode = xtal1+pll16x
  _clkfreq = 80_000_000


  'LCD is 480x272 giving 30x17 tiles
  cols = 30
  rows = 17
  screensize = cols * rows

CON  'Constants for Graphics Demo
  x_tiles = 22   'width of graphics area
  y_tiles = 13   'height of graphics area 
  GraphicsBytes=x_tiles*y_tiles*64
  _stack = ( GraphicsBytes + 100) >> 2   'accomodate display memory and stack  

  paramcount = 14       
  bitmap_base = $8000-GraphicsBytes'$5000    'these are the same because not double buffering...
  display_base = $8000-GraphicsBytes'$5000

  lines = 5
  thickness = 2

  GraphicsXOffset=1
  GraphicsYOffset=3

CON
    'number of custom 16x16 characters              
  nuchars = 1+4*3   '!!!!! you must have the correct # here for alignment later  

VAR

  long  lcd_status      'status: off/visible/invisible  read-only       (5 contiguous longs)
  long  lcd_enable      'enable: off/on                 write-only
  long  lcd_videobase   'video base @word               write-only
  long  lcd_colorbase   'color base @long               write-only
  long  lcd_backlight   'Used to control LCD backlight

  word  screen[screensize]
  long  col, row, color
  long  boxcolor,ptr

  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]

var 'vars for Graphics Demo
  long  mousex, mousey
  long touchx, touchy
  'long touchx0,touchdx,touchy0,touchdy   'cal factors
  long CurrentColor, bLineMode, bStillDown, LastX, LastY

VAR
      'for custom characters
  word user_charbase

OBJ
  lcd : "PSB_LcdDriver"  'Modified to drive 4.3" TFT
  gr    : "graphics"
  mouse : "mouse"
  i2c: "PSB_I2CDriver"  

pub Main
  'Prepare user characters for display (need to move so that last six bits of address are all zeroes!)
  '64 byte align the user characters
  user_charbase := @uchar & $FFC0 'destination
  'user_charbase_offset := user_charbase-@uchar  
  longmove(user_charbase,@uchar,16*nuchars)
  
  'Start up Display
  'clear screen
  print($100)
  'start driver
  lcd_enable:=1
  lcd_videobase := @screen
  lcd_colorbase := @vgacolors
  lcd_backlight:=3  '1=bright, 16=medium, 32=dark
  lcd.start(@lcd_status)

  i2c.initialize
  repeat 20     'init touch controller
    i2c.GetTouchX
 
  'CalibrateTouchScreen  'NOTE Uncomment out this line to do the touchscreen calibration !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


  Paint

  repeat

PUB Paint|dx,dy,GraphicsColor,xc,yc,tx,ty

  print($100)
  col:=9
  PrintStr(string("PSB Paint!"))

  'init colors
  GraphicsColor:=16 'Using this color for graphics colors
  'vgacolors[GraphicsColor]:=$0CFCCC00  'white, blue, magenta, black
  
  'init tile screen
  repeat dx from 0 to x_tiles - 1
    repeat dy from 0 to y_tiles - 1
      screen[(dy+GraphicsYOffset) * cols + dx+GraphicsXOffset] := display_base >> 6 + dy + dx * y_tiles +GraphicsColor<<10


  'start and setup graphics
  gr.start
  gr.setup(x_tiles, y_tiles, x_tiles*8, y_tiles*8, bitmap_base)

  gr.color(2)
  gr.width(3)

    'Center of graphics area:  xc= GraphicsXOffset*16+x_tiles*16/2
    '                          yc= GraphicsYOffset*16+y_tiles*16/2
  xc:=GraphicsXOffset*16+x_tiles*16/2
  yc:=GraphicsYOffset*16+y_tiles*16/2

  SetColor(1)
  SetLineMode(true) 'select line or dot mode
  bStillDown:=false

  
  'Draw Color Buttons
  Bitmap2Bit(@ColorButton, 24, 3, 2, 2, 17)
  Bitmap2Bit(@ColorButton, 27, 3, 2, 2, 18)
  Bitmap2Bit(@ColorButton, 24, 6, 2, 2, 19)
  Bitmap2Bit(@ColorButton, 27, 6, 2, 2, 20)
  'Draw Fill/Clear Screen buttons 
  Bitmap2Bit(@ColorButton, 24, 9, 2, 2, 21)  'fill
  Bitmap2Bit(@ColorButton, 27, 9, 2, 2, 22)  'clear
  'Draw Line/Dot Mode Buttons
  Bitmap2Bit(@LineButton, 24, 12, 2, 2, 23)  'fill
  Bitmap2Bit(@DotsButton, 27, 12, 2, 2, 24)  'clear  

  WaitStylusUp  'Don't start to draw until stylus is up   
  repeat
    WaitStylusDown(true)
    'touchx:=(touchx-touchx0)/touchdx
    'touchy:=(touchy-touchy0)/touchdy

    'See if any buttons were pushed
    tx:=touchx/16
    if tx>(GraphicsXOffset+x_tiles+1)
      'A button may have been pushed
      ty:=touchy/16
      CheckButtons(tx,ty)
    else
      'draw point    

       
      dx:=touchx-xc
      dy:=yc-touchy
      if bLineMode==true
        if bStillDown==true
            gr.line(dx,dy)
        else
          gr.plot(dx,dy)
      else
        gr.plot(dx,dy)
      LastX:=dx
      LastY:=dy
      bStillDown:=true 

PUB SetLineMode(bMode) 'select line or dot mode  
  if bMode==true
    bLineMode:=true
    vgacolors[23]:=$FF0C9C0C    'FF0C9C0C 
    vgacolors[24]:=$FF0C000C 
  else
    bLineMode:=false
    vgacolors[23]:=$FF0C000C 
    vgacolors[24]:=$FF0C9C0C

     
PUB SetColor(c) 'set current color and adjust buttons
  CurrentColor:=c
  gr.color(3-c)
  if c==0
    vgacolors[17]:=$FFFF9C0C
  else
    vgacolors[17]:=$FFFF000C
  if c==1
    vgacolors[18]:=$C0C09C0C
  else
    vgacolors[18]:=$C0C0000C
  if c==2
    vgacolors[19]:=$30309C0C
  else
    vgacolors[19]:=$3030000C
  if c==3
    vgacolors[20]:=$00009C0C
  else
    vgacolors[20]:=$0000000C    
      
      'long    $F0FF9C0C      '17 'Color0  white
      'long    $F0C09C0C       '18 Color1  red
      'long    $F030000C       '19 Color2  green
      'long    $F000000C       '20 Color3  black
                            
PUB CheckButtons(xTile,yTile)|column,i
  'see if stylus pushed on a button
  if xTile>23 and xTile<29
    'May have been a button
    if xTile<26
      'first column
      if yTile==3 or yTile==4  'White
        SetColor(0)
      if yTile==6 or yTile==7 'Green
        SetColor(2)
      if yTile==9 or yTile==10 'Fill screen
        i:=3-CurrentColor
        i:=i<<6+i<<4+i<<2+i
        bytefill(bitmap_base,i,GraphicsBytes)
      if yTile==12 or yTile==13 'Line Mode
        SetLineMode(true) 
    if xTile>26
      'Second column
      if yTile==3 or yTile==4  'Red
        SetColor(1)
      if yTile==6 or yTile==7 'Black
        SetColor(3)
      if yTile==9 or yTile==10 'Clear screen     
        gr.clear
      if yTile==12 or yTile==13 'Dot Mode
        SetLineMode(false)     

PUB CalibrateTouchScreen|TLX,TLY,TRX,TRY,TBLX,TBLY,TBRX,TBRY  'Calibrate the touchscreen by making people press on certain spots
  repeat 
    print($100)
    row:=6
    col:=4
    PrintStr(string("Calibrate Touch Screen"))
    row:=12
    col:=1
    PrintStr(string("(Tap and release each point)"))

    'Calibrate top-left corner
    WaitStylusUp
    Waitcnt(cnt+clkfreq/2) 
    Bitmap1Bit(@CalibrateCursor, 0, 0, 1, 1, 0)
    row:=0
    col:=3
    PrintStr(string(186,144,144,170," Press Here"))
    GetTouchCal
    TLX:=touchx
    TLY:=touchy
   
    'Calibrate top-right corner
    WaitStylusUp
    Waitcnt(cnt+clkfreq/2)
    print($100)      
    Bitmap1Bit(@CalibrateCursor, cols-1, 0, 1, 1, 0)
    row:=0
    col:=10
    PrintStr(string("Press Here ",169,144,144,187))
    GetTouchCal
    TRX:=touchx
    TRY:=touchy

    'Calibrate bottom-left corner
    WaitStylusUp
    Waitcnt(cnt+clkfreq/2)
    print($100)
    Bitmap1Bit(@CalibrateCursor, 0, rows-2, 1, 1, 0)
    row:=rows-2
    col:=3
    PrintStr(string(186,144,144,170," Press Here"))
    GetTouchCal
    TBLX:=touchx
    TBLY:=touchy

    'Calibrate bottom-right corner
    WaitStylusUp
    Waitcnt(cnt+clkfreq/2)
    print($100)      
    Bitmap1Bit(@CalibrateCursor, cols-1, rows-2, 1, 1, 0)
    row:=rows-2
    col:=10
    PrintStr(string("Press Here ",169,144,144,187))
    GetTouchCal
    TBRX:=touchx
    TBRY:=touchy

    'Calculate touchscreen cal factors
    'horizontal pixels between touchpoints = 16*(cols-1)
    'vertical pixels between touchpoints=16*(rows-2)
    touchdx:=((TRX+TBRX)-(TLX+TBLX))/(32*(cols-1))
    touchdy:=((TBLY+TBRY)-(TLY+TRY))/(32*(rows-2))
    touchx0:=((TLX+TBLX)/2)-8*touchdx
    touchy0:=((TLY+TRY)/2)-16*touchdy

    
    'Now show cross in center and see if cal is good
    WaitStylusUp
    Waitcnt(cnt+clkfreq/2)
    print($100)      
    Bitmap1Bit(@CalibrateCursor, 15, 8, 1, 1, 0)
    row:=5
    col:=4
    PrintStr(string("Press Here ",5))
    GetTouchCal

    'Calculate if cal good
    'test x is at 15*16+7=247
    'test y is at 8*16+15=143
    if (||(((touchx-touchx0)/touchdx)-247)<5) and (||(((touchy-touchy0)/touchdy)-143)<5) 
      'Good Cal
      Return
    else
      Waitcnt(cnt+clkfreq/2)
      print($100)      
      row:=5
      col:=4
      PrintStr(string("Calibration Failed."))
      Waitcnt(cnt+clkfreq)
      row:=15
      col:=0      
      PrintStr(string(" X="))
      dec((touchx-touchx0)/touchdx)
      PrintStr(string(" Y="))
      dec((touchy-touchy0)/touchdy)
      
{    
  repeat
    repeat until ina[27]==0 
    row:=15
    col:=0      
    PrintStr(string("Down:  X="))
    dec((i2c.GetTouchX-touchx0)/touchdx)
    PrintStr(string(" Y="))
    dec((i2c.GetTouchY-touchy0)/touchdy)
    print(32)
}
      
PUB GetTouchCal|tx,ty  'take 5 samples and average
    tx~
    ty~    
    repeat 5
      WaitStylusDown(false)
      tx+=touchx
      ty+=touchy
    touchx:=tx/5
    touchy:=ty/5

PUB WaitStylusUp
  'Wait for stylus to be up
  repeat
    waitcnt(cnt+clkfreq/100)  
    if ina[27]<>0
    return

    
PUB WaitStylusDown(bCalibrated)|tx,ty,z1,z2,tz
  'Wait for stylus to be down then update touchx, touchy and return
  repeat
    waitcnt(cnt+clkfreq/1000)  
    if ina[27]==0
      tx:=i2c.GetTouchX
      ty:=i2c.GetTouchY
      z1:=i2c.GetTouchZ1
      z2:=i2c.GetTouchZ2
      'take another x,y reading and average
      tx+=i2c.GetTouchX
      ty+=i2c.GetTouchY
      tx>>=1
      ty>>=1
      tz:=tx*(z2-z1)/z1
      waitcnt(cnt+clkfreq/1000) 
      if (tz<57500) and (tz>30000) and (ina[27]==0 ) 
        if bCalibrated==true
          'calculate calibrated pixel
          touchx:=(tx-touchx0)/touchdx
          touchy:=(ty-touchy0)/touchdy
        else
          'set uncalibrated data
          touchx:=tx
          touchy:=ty
        return
      else
        bStillDown:=false
    else
      bStillDown:=false        


PRI Bitmap1Bit(pBitmap, xPos, yPos, xSize, ySize, clr)|c,i,j,BmpAddress
  row:=yPos
  col:=xPos
  c:=0
  BmpAddress:=pBitmap+user_charbase-@uchar
  repeat j from 0 to (ySize-1) step 2
    repeat i from 0 to (xSize-1)
      screen[row * cols + col] := (clr<<1+1) << 10 + BmpAddress>>6 +c
      screen[(row+1) * cols + col] := (clr<<1) << 10 + BmpAddress>>6 +c
      c++
      col++
    row+=2
    col:=xPos

PRI Bitmap2Bit(pBitmap, xPos, yPos, xSize, ySize, clr)|c,i,j,BmpAddress
  row:=yPos
  col:=xPos
  c:=0
  BmpAddress:=pBitmap+user_charbase-@uchar
  repeat j from 0 to (ySize-1)
    repeat i from 0 to (xSize-1)
      screen[row * cols + col] := (clr) << 10 + BmpAddress>>6 +c
      'Print2Bit(c,clr,pBitmap)
      c++
      col++
    row++
    col:=xPos

PRI PrintStr(stringptr)

'' Print a zero-terminated string

  repeat strsize(stringptr)
    print(byte[stringptr++])

PUB print(c) | i, k

  case c
    $00..$FF:           'character?
      k := color << 1 + c & 1
      i := k << 10 + $200 + c & $FE
      screen[row * cols + col] := i
      screen[(row + 1) * cols + col] := i | 1
      if ++col == cols
        newline

    $100:               'clear screen?
      wordfill(@screen, $220, screensize)
      col := row := 0

    $108:               'backspace?
      if col
        col--

    $10D:               'return?
      newline

    $110..$11F:         'select color?
      color := c & $F


' New line

PRI newline : i

  col := 0
  if (row += 2) == rows
    row -= 2
    'scroll lines
    repeat i from 0 to rows-3
'      wordmove(@screen[i*cols], @screen[(i+2)*cols], cols)
    'clear new line
'    wordfill(@screen[(rows-2)*cols], $200, cols<<1)


' Data

PRI dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    print("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      print(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      print("0")
    i /= 10

PRI hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    print(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))
    
DAT 'Default Touchscreen Calibrations
touchx0 long 2713
touchdx long 123
touchy0 long 4590
touchdy long 206

DAT  'Bitmap data
padding LONG  7[16] 'alignment padding for the following user defined characters
uchar long

CalibrateCursor long 
file "calibrateCursor.dat"  '1 tile
ColorButton long
file "button1.dat"  '2x2=4 tiles
LineButton long
file "button2.dat"  '2x2=4 tiles
DotsButton long
file "button3.dat"  '2x2=4 tiles


DAT

vgacolors               long
                        long    $F00CF00C       'blue background with yellow letters
                        long    $F0F00C0C
                        long    $08A808A8       'green
                        long    $0808A8A8
                        long    $50005000       'blue
                        long    $50500000
                        long    $FC00FC00       'white
                        long    $FCFC0000
                        long    $FF80FF80       'red/white
                        long    $FFFF8080
                        long    $FF20FF20       'green/white
                        long    $FFFF2020
                        long    $FF28FF28       'cyan/white
                        long    $FFFF2828
                        long    $00A800A8       'grey/black
                        long    $0000A8A8
                        long    $FFC03000      '16'Colors for Graphics
spcl                    long    $F0FF9C0C       '17 'Color0  white
                        long    $F0C09C0C        '18 Color1  red
                        long    $F030000C      '19 Color2  green
                        long    $F000000C      '20 Color3  black
                        long    $FF0C9C0C        '21 fill screen
                        long    $000C9C0C        '22 clear screen
                        long    $FF0C9C0C        '23 line mode
                        long    $FF0C9C0C        '24 dots mode 
                        long    0

